/* A program to do clustering
   written by Masakazu Yamamoto.
*/

#include <khebios.h>
#include <stdlib.h>
#include <stdio.h>

#define FORWARD_SPEED 10

#define ARM_ST_POS 140


/* Pay attention: the parameters BACK_DISCHARGED and BACK_CHARGED depend */
/* on the gripper type (old or new) and on the robots IR sensors (the    */
/* distinction range play a crucial role for the final position between  */
/* robot and seed. 							 */
/* The following robot -gripper pairs are proposed:			 */
/* GR1 -> 028-06, GR2 -> 028-07, GR3 -> P05-43, GR4 -> P19-82, 		 */
/* GR5 -> 97406, GR6 -> 97408, GR7 -> P45-112, GR8 -> P19-95, 		 */
/* GR9 -> 96394, GR10 -> 97410						*/

/* for gripper LAMI GR1 */
/*#define ARM_GRIP_POS 250
#define ARM_DEGRIP_POS 248
#define BACK_DISCHARGED -550
#define BACK_CHARGED -250*/

/* for gripper LAMI GR2*/
/*#define ARM_GRIP_POS 240 
#define ARM_DEGRIP_POS 238  
#define BACK_DISCHARGED -550
#define BACK_CHARGED -250*/

/* for gripper LAMI GR3*/
/*#define ARM_GRIP_POS 245 
#define ARM_DEGRIP_POS 243  
#define BACK_DISCHARGED -550
#define BACK_CHARGED -250*/

/* for gripper LAMI GR4*/
/*#define ARM_GRIP_POS 255
#define ARM_DEGRIP_POS 253
#define BACK_DISCHARGED -600
#define BACK_CHARGED -320 */

/* for gripper LAMI GR5, GR6*/
/*#define ARM_GRIP_POS 250
#define ARM_DEGRIP_POS 248
#define BACK_DISCHARGED -650
#define BACK_CHARGED -380 */

/* for gripper LAMI GR7*/
#define ARM_GRIP_POS 244
#define ARM_DEGRIP_POS 242
#define BACK_DISCHARGED -600
#define BACK_CHARGED -320 

/* for gripper LAMI GR8*/
/*#define ARM_GRIP_POS 250
#define ARM_DEGRIP_POS 248
#define BACK_DISCHARGED -600
#define BACK_CHARGED -300 */

/* for gripper LAMI GR9*/
/*#define ARM_GRIP_POS 250
#define ARM_DEGRIP_POS 248
#define BACK_DISCHARGED -600
#define BACK_CHARGED -320 */

/* for gripper LAMI GR10*/
/*#define ARM_GRIP_POS 252
#define ARM_DEGRIP_POS 250
#define BACK_DISCHARGED -650
#define BACK_CHARGED -380*/ 


#define MAX_SPEED 20
#define ROTATION_SPEED 5
#define APPROACH_SPEED 2
#define OBJECT 800

#define DO_LOCK_AVOID 1
#define _DO_LIMIT_SPEED_

/* #define _CNTER_C2_ */

#ifndef _CNTER_C2_
#define _CNTER_C3_
#endif


/* for P05-043 */
/*const float PROX_COEF[]={2.7, 1.1, 1.0, 1.0, 1.6, 1.3, 1.0, 1.0};*/ 

/* for 028-06 */
/*const float PROX_COEF[]={1.1, 1.0, 1.0, 1.1, 1.0, 1.0, 1.1, 1.0};*/

/* for 028-07 */
/*const float PROX_COEF[]={1.3, 1.0, 1.0, 1.0, 1.1, 1.2, 2.0, 1.2};*/

/* for P19-82*/
/*const float PROX_COEF[] = {1.8, 1.0, 1.4, 2.0, 1.6, 1.4, 1.0, 1.1};*/

/* for P19-95 */
/*const float PROX_COEF[] = {1.0, 1.0, 1.6, 1.8, 1.1, 1.8, 1.6, 2.3};*/

/* for P45-112*/
/*const float PROX_COEF[] = {1.0, 1.1, 1.2, 1.2, 1.4, 2.2, 1.0, 1.0};*/

/* for 95217*/
const float PROX_COEF[] = {1.9, 1.0, 1.7, 3.5, 2.1, 1.9, 2.1, 2.6};

/* for 96394*/
/*const float PROX_COEF[] = {1.8, 1.0, 1.4, 1.5, 1.7, 1.9, 1.0, 1.3};*/

/* for 97406 */
/*const float PROX_COEF[] = {2.6, 1.3, 3.4, 7.1, 1.0, 3.1, 2.3, 1.3};*/

/* for  97408 */
/*const float PROX_COEF[] = {2.4, 1.6, 6.8, 2.7, 1.0, 2.3, 3.7, 6.8};*/ 

/* for  97410 */
/*const float PROX_COEF[] = {2.6, 2.1, 1.6, 2.3, 1.6, 1.0, 1.0, 1.3}; */


#include 	"rout_pinza.c"

bool	charged;
int	locked;  /* value (0-10) to detect that khepera can't move
                     with speed by Braitenberg */

void initial()
{
  bios_reset();
  ser_reset();
  mot_reset();
  sens_reset();
  var_reset();
  tim_reset();

  ser_config(6);		/* config the serial line Baudrate to 38400 */
  mot_config_pid(0,3500,800,100); /* set the PID parameters motor 0 */
  mot_config_pid(1,3500,800,100); /* set the PID parameters motor 1 */

  elevate_new(ARM_ST_POS,50,7000);
}

void print_time()
{
  int32 t;
  t = tim_get_ticcount();
  printf("%ld hour, %ld min, %ld sec\n",
	 t/3600000,(t/60000)%60,(t/1000)%60); /* t%1000 [msec] */
}  

void blink_LED()
{
  for(;;)
    {
      tim_suspend_task(2000);	/* wait 2 seconds */
      var_change_led(0);	/* change state of led 0 */
    }
}

int lock_avoid(int32 speed0, int32 speed1)
{
/*  int i;*/
  int32	start_locked_time;
  int32 status0,status1;
/*  int32 Sensor[8]; */

  status0 = mot_get_status(0) % 65536;
  status1 = mot_get_status(1) % 65536;
  if(status0 > 32767) 
    status0 = 65536 - status0;
  if(status1 > 32767) 
    status1 = 65536 - status1;

  if(status0 > iabs(speed0 / 2) || status1 > iabs(speed1 / 2) )
    {
      locked ++;
/*      printf("###Locked:%d[%ld %ld]\n",locked,status0,status1);  */
    }
  else if(locked > 0)
    locked --;

  if(locked > 30 && DO_LOCK_AVOID) /* for KT2+LG 30 , other 20 */
    {
      mot_new_speed_pid(0, -speed0);
      mot_new_speed_pid(1, -speed1);
      printf("## Locked! L:%ld\tR:%ld\n",speed0,speed1);
      start_locked_time=tim_get_ticcount();
      while(tim_get_ticcount() < start_locked_time + 6000/(speed0+speed1))
	{
#ifdef __COMMENT__	  
	  for( i=0; i<8; i++) /* Sensor activation */
	    {
	      Sensor[i] = sens_get_reflected_value(i);
	      if (Sensor[i] > 900)
		{
		  mot_new_speed_pid(0,0);
		  mot_new_speed_pid(1,0);
		  locked =0;
		  return(2); /* termination after locked and 
			        detect something */
		}
	    }
#endif
	}
      mot_new_speed_pid(0, speed0);
      mot_new_speed_pid(1, speed1);
      locked =0;
      return(1); /* termination after locked */
    } /* end of if(locked > 5)*/
  return(0); /* normal termination (not locked)*/
}

void braiten(int32 tim) /* tim [msec]*/
{
/*  int32 Interconn[16] = {4,4,6,-18,-15,-5,5,3, -5,-15,-18,6,4,4,3,5};	*/
  int32 Interconn[16] = {4,4,6,-20,-17,-6,5,3, -6,-17,-20,6,4,4,3,5};	
	/* interconnection matrix between IR proximity sensors and motors */
  int32	Offset[2] = {FORWARD_SPEED,FORWARD_SPEED};
	/* default forward speed */
  int32	Sensor[8], Motor[2];
  int i,j;
  int32	start_time;
  static int32 cycle = 0;

  printf("Start Braitenberg! cycle[%ld]\n",cycle++);
  print_time();

  start_time = tim_get_ticcount();
  do
    {
      for( i=0; i<8; i++)	/* Sensor activation */
	{
	  Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];
	}

      for( j=0; j<2; j++) 
	{
	  for( i=0, Motor[j]=0; i<8; i++)
	    {			/* Connections sensors-motors */
	      Motor[j] += Sensor[i]*Interconn[(j*8)+i];
	    }
	  Motor[j] = Motor[j]/400 + Offset[j];    /* Global gain and Offset */
	}
#ifdef _DO_LIMIT_SPEED_
      for( j=0; j<2; j++)
	{
	  if(Motor[j] < -MAX_SPEED)
	    {
	      Motor[(j+1)%2] =(int32)(Motor[(j+1)%2]*(double)(-MAX_SPEED/Motor[j]));
	      Motor[j]= -MAX_SPEED;
	    }
	  if(Motor[j] > MAX_SPEED)
	    {
	      Motor[(j+1)%2] = (int32)(Motor[(j+1)%2]*(double)(MAX_SPEED/Motor[j]));
	      Motor[j]=  MAX_SPEED;
	    }
	}
#endif
      mot_new_speed_pid(0, Motor[0]); /* Set motor speed */
      mot_new_speed_pid(1, Motor[1]);

      tim_suspend_task(100); 
      if(lock_avoid(Motor[0],Motor[1])==1)
	{
	  turn(180);
	}
    }
  while ( tim_get_ticcount() < start_time + tim);
  locked = 0;
/*  mot_new_speed_pid(0,0);
  mot_new_speed_pid(1,0);*/

  printf("End of braitenberg.\n");
  print_time();
}

void search_object()
{
  int32	Sensor[8];
  int i;

  printf("Start Searching! \n");
  print_time();
  
  mot_new_speed_pid(0,FORWARD_SPEED);
  mot_new_speed_pid(1,FORWARD_SPEED);
  do
    {
      for( i=0; i<6; i++)	/* Sensor activation */
	Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];
      
      if(Sensor[2]>OBJECT-100 || Sensor[3]>OBJECT-100 || Sensor[1]>OBJECT  
	 || Sensor[4]>OBJECT || Sensor[0]>OBJECT || Sensor[5]>OBJECT)
	break;

      tim_suspend_task(50); 
      if(lock_avoid(FORWARD_SPEED,FORWARD_SPEED)==1)
	{
	  locked = 0;
	  turn(45);
	  mot_new_speed_pid(0,FORWARD_SPEED);
	  mot_new_speed_pid(1,FORWARD_SPEED);
	  tim_suspend_task(100);
	}
    }
  while (1);
  locked = 0;

  mot_new_speed_pid(0,0);
  mot_new_speed_pid(1,0);

  for(i=0;i<6;i++)
    printf("C%d:%4ld\t",i,Sensor[i]);
  printf("\n");

  printf("End of searching.\n");
  print_time();

}

int disting()
{
  int32	Sensor[8];
  int i,decide,count,loop_count, old_count;
  int turn_dir; /* direction of turning 0:counter clockwise 1:clockwise */
  int biggest;
  int32 tmp_time;

  turn_dir = -1;
  decide= 0; /* 0:unknown  1:little object  2:Khepera  3:Wall  4: 2or3 */
  count = 0;
  loop_count = 0;
  old_count = -10;

  do  /* loop for centering */
    {
      for( i=0; i<6; i++)	/* Sensor activation */
	Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];

      biggest = 0;
      for( i=1; i<6; i++)
	if (Sensor[biggest] < Sensor[i])
	  biggest = i;

/*   if(Sensor[0]/2+2*Sensor[1]+4*Sensor[2] > 4*Sensor[3]+2*Sensor[4]+Sensor[5]/2 + 50)*/
      if(biggest < 3)
	{
	  if(turn_dir !=0 )
	    {
	      mot_new_speed_pid(0,-2);
	      mot_new_speed_pid(1,2);
	      turn_dir=0; /* counter clockwise */
	    }
	}
/*   else if(Sensor[0]/2+2*Sensor[1]+4*Sensor[2] + 50 < 4*Sensor[3]+2*Sensor[4]+Sensor[5]/2) */
      else
	if(turn_dir !=1 )
	  {
	    mot_new_speed_pid(0,2);
	    mot_new_speed_pid(1,-2);
	    turn_dir=1; /* clockwise */
	  }
      for(i=0;i<6;i++)
	printf("C%d:%4ld\t",i,Sensor[i]);
      printf("\n");

      if(!decide) /* There is no decision */
	{
	  if(!turn_dir) /* if counter clockwise */
	    {
	      if((Sensor[1]>=1023 && Sensor[0]+Sensor[2]<30)
		 ||(Sensor[2]>=1023 && Sensor[1]+Sensor[3]<30) )
		{
		  if(old_count == loop_count -1 || !count)
		    {
		      count ++;
		      printf("*count:%d\n",count); 
		      old_count = loop_count;
		      if(count >2)
			decide=1;	/* decide little object */
		    }
		}
	      else if ((Sensor[1] >=1023 && abs(Sensor[0]-Sensor[2])<150 
			&& Sensor[0]+Sensor[2] > 200) ||
		       (Sensor[2] >=1023 && abs(Sensor[1]-Sensor[3])<150 
			&& Sensor[1]+Sensor[3] > 200))
		{
		  decide =4;	/* maybe another Khepera (or wall) */
		  printf("Khepera or Wall\n");
		  return(4);
		}
	    }
	  else /* clockwise */
	    {
	      if((Sensor[3]>=1023 && Sensor[2]+Sensor[4]<30)
		 ||(Sensor[4]>=1023 && Sensor[3]+Sensor[5]<30) )
		{
		  if(old_count == loop_count -1 || !count)
		    {
		      count ++;
		      printf("*count:%d\n",count); 
		      old_count = loop_count;
		      if(count >2)
			decide=1;	/* decide little object */
		    }
		}		
	      else if ((Sensor[3]>=1023 && abs(Sensor[2]-Sensor[4])<150 
			&& Sensor[2]+Sensor[4] > 200) ||
		       (Sensor[4]>=1023 && abs(Sensor[3]-Sensor[5])<150 
			&& Sensor[3]+Sensor[5] > 200))
		{
		  decide =4;	/* maybe another Khepera (or wall) */
		  printf("Khepera or Wall\n");
		  return(4);
		}
	    }
	}/* end of if(!decide) */
      lock_avoid(2,2); /* if both motor speed are zero -> locked */
      tim_suspend_task(50);
      loop_count++;
    }
  while(abs(Sensor[0]/2 + Sensor[1] + Sensor[2] - Sensor[3]- Sensor[4] - Sensor[5]/2) > 100 && loop_count < 100);
		/* end of loop for centering */
  locked = 0;

  mot_new_speed_pid(0,0);
  mot_new_speed_pid(1,0);

  for(i=0;i<6;i++)
    printf("C%d:%4ld\t",i,Sensor[i]);
  printf("\n");
  
  if (Sensor[2]<100 && Sensor[3]<100 )
    {
      printf("Missing !\n");	/* it cause also when there are objects both side of robot */
      return 0;
    }
  
  if (Sensor[2]<1000 && Sensor[3]<1000 )
    {			/* After centering, if the object is far from robot, he aproach. */
      mot_new_speed_pid(0,2);
      mot_new_speed_pid(1,2);

      tmp_time = tim_get_ticcount();
      do
	{
	  for( i=0; i<8; i++) /* Sensor activation */
	    Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];
/*	      for(i=0;i<8;i++)
		printf("C%d:%4ld\t",i,Sensor[i]);
	      printf("\n");
*/
	  lock_avoid(2,2);
	  tim_suspend_task(50);
	  if(tim_get_ticcount() > tmp_time + 2000){
	    printf("Missing !!\n");
	    return 0;
	  }
	}
      while(Sensor[2] < 1000 && Sensor[3] < 1000 );
      locked = 0;

      mot_new_speed_pid(0,0);
      mot_new_speed_pid(1,0);
    } /* end of aproach */
      
  if(Sensor[1]+Sensor[4]> 300)
    {
      printf("I think it is WALL ...\n");
/*	  printf("CCCWWWC1:%ld,C2:%ld,C3:%ld,C4:%ld\n",Sensor[1],Sensor[2],Sensor[3],Sensor[4]); */
      decide = 3; /* even if there is already decision, it changes */
      return(3);
    }
  if(decide)
    {
      if(decide==1)
	printf("I think it is little OBJECT (by predecide)!\n");
      else
	printf("I think it is another Khepera (by predecide)!\n");
    }
  do{
    for( i=0; i<8; i++) /* Sensor activation */
      Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];
#ifdef _CNTER_C2_ 
    if(Sensor[0]+4*Sensor[1] > 4*Sensor[3]+Sensor[4])
#endif
#ifdef _CNTER_C3_ 
    if(Sensor[1]+4*Sensor[2] > 4*Sensor[4]+Sensor[5])
#endif
      {
	mot_new_speed_pid(0,-2);
	mot_new_speed_pid(1,2);
      }
    else
      {
	mot_new_speed_pid(0,2);
	mot_new_speed_pid(1,-2);
      }
    for(i=0;i<8;i++)
      printf("C%d:%4ld\t",i,Sensor[i]);
    printf("\n");
  }
#ifdef _CNTER_C2_ 
  while(abs(Sensor[0]+Sensor[1] - (Sensor[3]+Sensor[4])) > 50);
#endif
#ifdef _CNTER_C3_ 
  while(abs(Sensor[1]+Sensor[2] - (Sensor[4]+Sensor[5])) > 50);
#endif

  mot_new_speed_pid(0,0);
  mot_new_speed_pid(1,0);

  count = 0;
  for(i=0;i<5;i++){
#ifdef _CNTER_C2_ 
    if(Sensor[2] < 1000){
      printf("Missing !!!\n");
      return 0;
    }
    if(Sensor[1]+Sensor[3]> 200)
#endif
#ifdef _CNTER_C3_ 
    if(Sensor[3] < 1000){
      printf("Missing !!!\n");
      return 0;
    }
    if(Sensor[2]+Sensor[4]> 200 && Sensor[3] > 1000)
#endif
      {
	printf("[K2] C1:%ld,C2:%ld,C3:%ld,C4:%ld\n"
	       ,Sensor[1],Sensor[2],Sensor[3],Sensor[4]);
      }
    else
      {
	printf("[L2] C1:%ld,C2:%ld,C3:%ld,C4:%ld\n"
	       ,Sensor[1],Sensor[2],Sensor[3],Sensor[4]);
	count++;
      }
    tim_suspend_task(50);
  } /* end of for(i=0;i<9;i++)*/
  if(count >= 5){
    printf("I think it is a little OBJECT !\n");
    decide = 1;
  }else{
    printf("I think it is another Khepera !\n");
    /* decide = 2; */
    return 2;
  }
  
  if(decide)  /* i.e. decide = 1 */
    {
      do{			/* loop for centering (to make it easy to handle object) */
	for( i=0; i<6; i++)	/* Sensor activation */
	  Sensor[i] = sens_get_reflected_value(i)*PROX_COEF[i];
    
	if(Sensor[0]/2+2*Sensor[1]+4*Sensor[2] > 4*Sensor[3]+2*Sensor[4]+Sensor[5]/2)
	  {
	    mot_new_speed_pid(0,-2);
	    mot_new_speed_pid(1,2);
	  }
	else
	  {
	    mot_new_speed_pid(0,2);
	    mot_new_speed_pid(1,-2);
	  }
/*	for(i=0;i<6;i++)
	  printf("C%d:%4ld\t",i,Sensor[i]);
	printf("\n");
*/
      }
      while(abs(Sensor[0]/2+2*Sensor[1]+4*Sensor[2] - (4*Sensor[3]+2*Sensor[4]+Sensor[5]/2)) > 400);	
      mot_new_speed_pid(0,0);
      mot_new_speed_pid(1,0);
    }
  return(decide);
}

void elevate_object()
{
  int err;
  int serrage;
  err = 0;

  printf("CHARGE\n");

  mot_new_speed_pid(0, -APPROACH_SPEED);
  mot_new_speed_pid(1, -APPROACH_SPEED);

  do
    {
      tim_suspend_task(50);
      lock_avoid(-APPROACH_SPEED,-APPROACH_SPEED);
    }
  while(sens_get_reflected_value(0)*PROX_COEF[0] >= 1023 && 
	sens_get_reflected_value(1)*PROX_COEF[1] >= 1023);
  locked = 0;

  mot_new_speed_pid(0, 0);
  mot_new_speed_pid(1, 0);

  if(!move(BACK_CHARGED,10000))
    {
      degrip();
      if(elevate_new(ARM_GRIP_POS,15,3000)==FALSE)
	{
	  printf("bas failed\n");
	  elevate_new(ARM_ST_POS,0,0);
	  err=1;
	}

      grip(0,20,1000);
      tim_suspend_task(100);

      if(elevate_new(ARM_ST_POS,50,3000)==FALSE && err == 0)
	{
	  printf("haut failed\n");
	  degrip();
	  elevate_new(ARM_ST_POS,0,0);
	  err=1;
	}
      if(err == 0)
	{
	  if( (serrage=get_serrage()) < 30 || serrage>90)
	    {
	      printf("PARDON: taille: %d\n",serrage);
	      degrip();
	      charged = 0;
	    }
	  else
	    {
	      printf("OK: taille: %d\n",serrage);
	      charged = 1;
	    }
	}
      printf("succeed\n");
/*      turn(180);*/
    } /* end of if(!move(BACK_CHARGED)) */
  else
    {
      printf("Failed (I can't go more in this direction )\n");
    }
}

void put_object()
{
  int err;
  err = 0;
  printf("DECHARGE\n");

  mot_new_speed_pid(0, -APPROACH_SPEED);
  mot_new_speed_pid(1, -APPROACH_SPEED);

  do
    {
      tim_suspend_task(50);
      lock_avoid(-APPROACH_SPEED,-APPROACH_SPEED);
    }
  while(sens_get_reflected_value(0)*PROX_COEF[0] >= 1023 && 
	sens_get_reflected_value(1)*PROX_COEF[1] >= 1023);
  locked = 0;

  mot_new_speed_pid(0, 0);
  mot_new_speed_pid(1, 0);

  if(!move(BACK_DISCHARGED,10000))
    {
      if(elevate_new(ARM_DEGRIP_POS,15,3000)==FALSE)
	{
	  printf("bas failed\n");
	  elevate_new(ARM_ST_POS,0,0);
/*	  turn(180);*/
	  err=1;
	}
      else
	{
	  degrip();
	  tim_suspend_task(100);
      
	  charged = 0;
      
	  elevate_new(ARM_ST_POS,50,3000);
      
	  printf("decharge'\n");
      
/*	  turn(180);*/
	}      
    }/* end of if(!move(BACK_DISCHARGED)) */
  else
    {
      printf("Failed (I can't go more in this direction )\n");
    }
}

void main()
{
  int key;
  int dead_lock = 0;
  int32 time;

  initial();
  tim_new_task(blink_LED);

  for(;;)
    {
      time = tim_get_ticcount();
      search_object();		/* look for something */
      key = disting();		/* decide what it is*/
      if(key == 1)
	{
	  if(!charged)
	    elevate_object();
	  else
	    put_object();
	  braiten(3000);   /* braitenberg(3sec) */
	}
      else	
	braiten(2000);	/* braitenberg(2sec) */
      if(time + 3000 > tim_get_ticcount())
	{
	  printf("*** Dead lock %d\n",++dead_lock);
	}
      else
	dead_lock = 0;
      if(dead_lock >= 3)
	{
	  printf("I think I'm in dead lock...I'll go back\n");
	  if(move(BACK_DISCHARGED,10000))
	    printf("Failed (I can't go more in this direction )\n");
	  turn(90);
	  braiten(3000);
	  dead_lock = 0;
	}
    }
}
