/*
; test_radio.
; ===========

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:	KHEPERA
; Goal:		Test of the radio driver.
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#include	<khesys.h>
#include	<radio.h>
#include 	<stdlib.h>
#include 	<stdio.h>

uint32	vIDProcess[2];

/* Function 1 */
/* >>>>>>>>>> */

void function_1()
	{
	int32	status;
	uint8	vVersion, vRevision;

	status = radio_getVersion();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		vVersion = (uint8)(status >> 8);
		vRevision = (uint8)(status);
		RESERVE_COM
		printf("Version %d.%d\n", vVersion, vRevision);
		RELEASE_COM
		}
	}

/* Function 2 */
/* >>>>>>>>>> */

void function_2()
	{
	int32	status;
	uint8	buffer[] = {2, 5, 0,1,2,3,4};

	status = radio_sndBuffer(buffer);
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem %d\n", status);
		RELEASE_COM
		}
	}

/* Function 3 */
/* >>>>>>>>>> */

void function_3()
	{
	int32	status;
	uint8	buffer[1+1+16];
	uint32	i;

	status = radio_recBuffer(buffer);
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem %d\n", status);
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("ID of the sender    %d\n", buffer[0]);
		printf("Size of the message %d\n", buffer[1]);
		printf("Message ...         ");
		for (i = 0; i < buffer[1]; i++) printf("%d ", buffer[2+i]);
		printf("\n\n\n\n");
		RELEASE_COM
		}
	}

/* Function 4 */
/* >>>>>>>>>> */

void function_4()
	{
	int32	status;

	status = radio_getStatus();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("Status LMS = %d, MAI = %d, 10T = %d\n", (uint8)(status & 0x1), \
														(uint8)((status >> 1) & 0x1), \
														(uint8)((status >> 2) & 0x1));
		RELEASE_COM
		}
	}

/* Function 5 */
/* >>>>>>>>>> */

void function_5()
	{
	int32	status;

	status = radio_getIDNumber();
	if (status < 0)
		{
		RESERVE_COM
		printf("Network Communication Problem\n");
		RELEASE_COM
		}
	else
		{
		RESERVE_COM
		printf("ID of this turret %d\n", (uint8)status);
		RELEASE_COM
		}
	}

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */
 
void process_0()
	{

	for(;;)
		{
		tim_suspend_task(500);
		var_change_led(0);
		}
	}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * Launch a function.
 *
 */

void process_1()
	{
	
	for(;;)
		{
		tim_suspend_task(10000);
		function_3();
		}
	}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

void main(void)
	{
	int32	status;

	/*
	--------------------------|------------------------------------------------|
	*/

	static char	prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

	var_reset();
	radio_reset();

	status = install_task(prName_0, 800, process_0);
	if (status == -1) exit(0);
	vIDProcess[0] = (uint32)status;

	status = install_task(prName_1, 800, process_1);
	if (status == -1) exit(0);
	vIDProcess[1] = (uint32)status;
	}
