/*
 *	STDIO.H		Standard i/o include file
 */

#ifndef _STDIO_H
#define	_STDIO_H

/*extern	long	_chdis;		 channel of #display */
/*extern	long	_chkey;		 channel of #keyboard */

#include <stddef.h>
#include <types.h>

#define	FOPEN_MAX	(32)		/* maximum number of open streams */
#define	FILENAME_MAX	(128)		/* maximum filename size */
#define	BUFSIZ		(1024)		/* default buffer size */
#define	EOF		(-1)		/* end-of-file indicator */
#define	EOS		('\0')		/* end-of-string indicator */

/* lseek() origins */
#define	SEEK_SET	0		/* from beginning of file */
#define	SEEK_CUR	1		/* from current location */
#define	SEEK_END	2		/* from end of file */

/* FILE structure flags */
#define	_IOREAD		0x0001		/* file may be read from */
#define	_IOWRT		0x0002		/* file may be written to */
#define	_IOBIN		0x0004		/* file is in "binary" mode */
#define	_IODEV		0x0008		/* file is a character device */
#define	_IORW		0x0080		/* last i/o was 0:read/1:write */
#define	_IOFBF		0x0100		/* i/o is fully buffered */
#define	_IOLBF		0x0200		/* i/o is line buffered */
#define	_IONBF		0x0400		/* i/o is not buffered */
#define	_IOMYBUF	0x0800		/* standard buffer */
#define	_IOEOF		0x1000		/* EOF has been reached */
#define	_IOERR		0x4000		/* an error has occured */

typedef	struct				/* FILE structure */
	{
	short		_cnt;		/* # of bytes in buffer (must be signed!) */
	u_char		*_ptr;		/* current buffer pointer */
	u_char		*_base;		/* base of file buffer */
	u_short		_flag;		/* file status flags */
	u_long		_channel;	/* file handle (fos or xfos) */
	short		_bsiz;		/* buffer size (same type as _cnt) */
	u_char		_ch;		/* tiny buffer, for "unbuffered" i/o */
	}
	FILE;

#define	L_tmpnam	128
#define	TMP_MAX		1000

extern	FILE	_iob[];

extern	int	_doprnt	(const char *format, void *va, FILE *stream);
extern	int	_doscan	(FILE *stream, const char *format, ...);
extern	void	exit	(int status);
extern	void	_exit	(int status);
extern	int	fclose	(FILE *stream);
extern	int	fcloseall(void);
extern	FILE	*fdopen	(int handle, char *type);
extern	int	fflush	(FILE *stream);
extern	int	fgetc	(FILE *stream);
extern	char	*fgets	(char *s, int n, FILE *stream);
extern	FILE	*fopen	(const char *filename, const char *mode);
extern	int	fprintf	(FILE *stream, const char *format, ...);
extern	int	fputc	(int c, FILE *stream);
extern	int	fputs	(const char *s, FILE *stream);
extern	size_t	fread	(void *ptr, size_t size, size_t nobj, FILE *stream);
extern	FILE	*freopen(const char *filename, const char *mode, FILE *stream);
extern	int	fscanf	(FILE *stream, const char *format, ...);
extern	int	fseek	(FILE *stream, long offset, int origin);
extern	long	ftell	(FILE *stream);
extern	int	fungetc	(int c, FILE *stream);
extern	size_t	fwrite	(const void *ptr, size_t size, size_t nobj, FILE *stream);
extern	long	getl	(FILE *stream);
extern	char	*gets	(char *s);
extern	int	getw	(FILE *stream);
extern	char	*getwd	(char *pathname);
extern	int	printf	(const char *format, ...);
extern	long	putl	(long l, FILE *stream);
extern	int	puts	(const char *s);
extern	int	putw	(int w, FILE *stream);
extern	int	scanf	(const char *format, ...);
extern	int	sprintf	(char *s, const char *format, ...);
extern	int	sscanf	(char *s, const char *format, ...);
extern	int	ungetc	(int c, FILE *stream);
extern	int	vfprintf(FILE *stream, const char *format, char *arg);
extern	int	vfscanf	(FILE *stream, const char *format, char *arg);
extern	int	vprintf	(const char *format, char *arg);
extern	int	vscanf	(const char *format, char *arg);
extern	int	vsprintf(char *s, const char *format, char *arg);
extern	int	vsscanf	(char *s, const char *format, char *arg);

/* standard streams */
#define stdin	(&_iob[0])
#define stdout	(&_iob[1])
#define stderr	(&_iob[2])

/* stream macros */
#define clearerr(fp)	((void) ((fp)->_flag &= ~(_IOERR|_IOEOF)))
#define feof(fp)	((int) ((fp)->_flag & _IOEOF))
#define ferror(fp)	((int) ((fp)->_flag & _IOERR))
#define fileno(fp)	((int) ((fp)->_channel))

/* aliases */
#define	getc(s)		fgetc(s)
#define	putc(c,s)	fputc((c),(s))
#define	getchar()	fgetc(stdin)
#define	ungetchar(c)	fungetc((c),stdin)
#define	putchar(c)	fputc((c),stdout)

#endif	/* _STDIO_H */
